@@ -11,7 +11,8 @@ from xonsh.tools import ON_WINDOWS
 import pytest
 
 from xonsh.commands_cache import CommandsCache
-from xonsh.environ import Env, load_static_config, locate_binary, DEFAULT_ENSURERS, DEFAULT_VALUES, default_env
+from xonsh.environ import (Env, locate_binary, DEFAULT_ENSURERS, DEFAULT_VALUES,
+                           default_env, make_args_env)
 
 from tools import skip_if_on_unix
 
@@ -108,19 +109,15 @@ def test_swap():
     assert 'VAR3' not in env
 
 
-@pytest.mark.parametrize('s, exp, loaded',[
-    (b'{"best": "awash"}', {'best': 'awash'}, True), # works
-    (b'["best", "awash"]', {}, False), # fail
-    (b'{"best": "awash"', {}, False) # json fail
-])
-def test_load_static_config(s, exp, loaded, tmpdir, xonsh_builtins):
-    env = Env({'XONSH_SHOW_TRACEBACK': False})
-    xonsh_builtins.__xonsh_env__ = env
-    f = tmpdir.join('test_static_config')
-    f.write(s)
-    conf = load_static_config(env, str(f))
-    assert exp == conf
-    assert env['LOADED_CONFIG'] == loaded
+def test_swap_exception_replacement():
+    env = Env(VAR='original value')
+    try:
+        with env.swap(VAR='inner value'):
+            assert env['VAR'] == 'inner value'
+            raise Exception()
+    except Exception:
+        assert env['VAR'] == 'original value'
+    assert env['VAR'] == 'original value'
 
 
 @skip_if_on_unix
@@ -245,3 +242,15 @@ def test_no_lines_columns():
     finally:
         del os.environ['LINES']
         del os.environ['COLUMNS']
+
+
+def test_make_args_env():
+    obs = make_args_env(['script', '1', '2', '3'])
+    exp = {
+        'ARGS': ['script', '1', '2', '3'],
+        'ARG0': 'script',
+        'ARG1': '1',
+        'ARG2': '2',
+        'ARG3': '3',
+    }
+    assert exp == obs
