@@ -70,12 +70,21 @@ class CommandsCache(cabc.Mapping):
         else:
             return [name]
 
+    @staticmethod
+    def remove_dups(p):
+        ret = list()
+        for e in p:
+            if e not in ret:
+                ret.append(e)
+        return ret
+
     @property
     def all_commands(self):
         paths = builtins.__xonsh_env__.get('PATH', [])
-        pathset = frozenset(x for x in paths if os.path.isdir(x))
+        paths = CommandsCache.remove_dups(paths)
+        path_immut = tuple(x for x in paths if os.path.isdir(x))
         # did PATH change?
-        path_hash = hash(pathset)
+        path_hash = hash(path_immut)
         cache_valid = path_hash == self._path_checksum
         self._path_checksum = path_hash
         # did aliases change?
@@ -85,7 +94,7 @@ class CommandsCache(cabc.Mapping):
         self._alias_checksum = al_hash
         # did the contents of any directory in PATH change?
         max_mtime = 0
-        for path in pathset:
+        for path in path_immut:
             mtime = os.stat(path).st_mtime
             if mtime > max_mtime:
                 max_mtime = mtime
@@ -94,7 +103,7 @@ class CommandsCache(cabc.Mapping):
         if cache_valid:
             return self._cmds_cache
         allcmds = {}
-        for path in reversed(paths):
+        for path in reversed(path_immut):
             # iterate backwards so that entries at the front of PATH overwrite
             # entries at the back.
             for cmd in executables_in(path):
@@ -180,13 +189,34 @@ class CommandsCache(cabc.Mapping):
                       None)
         if cached:
             (path, alias) = self._cmds_cache[cached]
-            if not alias or ignore_alias:
-                return path
-            else:
+            ispure = path == pathbasename(path)
+            if alias and ignore_alias and ispure:
+                # pure alias, which we are ignoring
                 return None
+            else:
+                return path
         elif os.path.isfile(name) and name != pathbasename(name):
             return name
 
+    def is_only_functional_alias(self, name):
+        """Returns whether or not a command is only a functional alias, and has
+        no underlying executable. For example, the "cd" command is only available
+        as a functional alias.
+        """
+        _ = self.all_commands
+        return self.lazy_is_only_functional_alias(name)
+
+    def lazy_is_only_functional_alias(self, name):
+        """Returns whether or not a command is only a functional alias, and has
+        no underlying executable. For example, the "cd" command is only available
+        as a functional alias. This search is performed lazily.
+        """
+        val = self._cmds_cache.get(name, None)
+        if val is None:
+            return False
+        return val == (name, True) and \
+            self.locate_binary(name, ignore_alias=True) is None
+
     def predict_threadable(self, cmd):
         """Predicts whether a command list is able to be run on a background
         thread, rather than the main thread.
@@ -332,6 +362,23 @@ def predict_help_ver(args):
     return pred
 
 
+@lazyobject
+def HG_PREDICTOR_PARSER():
+    p = argparse.ArgumentParser('hg', add_help=False)
+    p.add_argument('command')
+    p.add_argument('-i', '--interactive', action='store_true', default=False,
+                   dest='interactive')
+    return p
+
+
+def predict_hg(args):
+    """Predict if mercurial is about to be run in interactive mode.
+    If it is interactive, predict False. If it isn't, predict True.
+    """
+    ns, _ = HG_PREDICTOR_PARSER.parse_known_args(args)
+    return not ns.interactive
+
+
 def default_threadable_predictors():
     """Generates a new defaultdict for known threadable predictors.
     The default is to predict true.
@@ -348,6 +395,7 @@ def default_threadable_predictors():
         'emacsclient': predict_false,
         'fish': predict_shell,
         'gvim': predict_help_ver,
+        'hg': predict_hg,
         'htop': predict_help_ver,
         'ipython': predict_shell,
         'ksh': predict_shell,
