@@ -1,4 +1,6 @@
 """Base completer for xonsh."""
+import collections.abc as cabc
+
 from xonsh.completers.path import complete_path
 from xonsh.completers.python import complete_python
 from xonsh.completers.commands import complete_command
@@ -15,6 +17,12 @@ def complete_base(prefix, line, start, end, ctx):
         paths = complete_path(prefix, line, start, end, ctx, False)
         return (out | paths[0]), paths[1]
     elif prefix == line:
-        return (complete_python(prefix, line, start, end, ctx) |
-                complete_command(prefix, line, start, end, ctx))
+        python_comps = complete_python(prefix, line, start, end, ctx)
+        if isinstance(python_comps, cabc.Sequence):
+            return (python_comps[0] |
+                    complete_command(prefix, line, start, end, ctx),
+                    python_comps[1])
+        else:
+            return (python_comps |
+                    complete_command(prefix, line, start, end, ctx))
     return set()
