@@ -3,7 +3,6 @@
 import os
 import re
 import sys
-import json
 import pprint
 import textwrap
 import locale
@@ -18,8 +17,6 @@ from xonsh.lazyasd import LazyObject, lazyobject
 from xonsh.codecache import run_script_with_cache
 from xonsh.dirstack import _get_cwd
 from xonsh.events import events
-from xonsh.foreign_shells import load_foreign_envs, load_foreign_aliases
-from xonsh.xontribs import update_context, prompt_xontrib_install
 from xonsh.platform import (
     BASH_COMPLETIONS_DEFAULT, DEFAULT_ENCODING, PATH_DEFAULT,
     ON_WINDOWS, ON_LINUX, os_environ
@@ -39,7 +36,7 @@ from xonsh.tools import (
     is_logfile_opt, to_logfile_opt, logfile_opt_to_str, executables_in,
     is_nonstring_seq_of_strings, pathsep_to_upper_seq,
     seq_to_upper_pathsep, print_color, is_history_backend, to_itself,
-    swap_values,
+    swap_values, ptk2_color_depth_setter
 )
 import xonsh.prompt.base as prompt
 
@@ -152,7 +149,6 @@ def DEFAULT_ENSURERS():
     'LC_MONETARY': (always_false, locale_convert('LC_MONETARY'), ensure_string),
     'LC_NUMERIC': (always_false, locale_convert('LC_NUMERIC'), ensure_string),
     'LC_TIME': (always_false, locale_convert('LC_TIME'), ensure_string),
-    'LOADED_CONFIG': (is_bool, to_bool, bool_to_str),
     'LOADED_RC_FILES': (is_bool_seq, csv_to_bool_seq, bool_seq_to_csv),
     'MOUSE_SUPPORT': (is_bool, to_bool, bool_to_str),
     'MULTILINE_PROMPT': (is_string_or_callable, ensure_string, ensure_string),
@@ -161,6 +157,7 @@ def DEFAULT_ENSURERS():
                 seq_to_upper_pathsep),
     'PRETTY_PRINT_RESULTS': (is_bool, to_bool, bool_to_str),
     'PROMPT': (is_string_or_callable, ensure_string, ensure_string),
+    'PROMPT_TOOLKIT_COLOR_DEPTH': (always_false, ptk2_color_depth_setter, ensure_string),
     'PUSHD_MINUS': (is_bool, to_bool, bool_to_str),
     'PUSHD_SILENT': (is_bool, to_bool, bool_to_str),
     'RAISE_SUBPROC_ERROR': (is_bool, to_bool, bool_to_str),
@@ -190,6 +187,7 @@ def DEFAULT_ENSURERS():
     'XONSH_ENCODING_ERRORS': (is_string, ensure_string, ensure_string),
     'XONSH_HISTORY_BACKEND': (is_history_backend, to_itself, ensure_string),
     'XONSH_HISTORY_FILE': (is_string, ensure_string, ensure_string),
+    'XONSH_HISTORY_MATCH_ANYWHERE': (is_bool, to_bool, bool_to_str),
     'XONSH_HISTORY_SIZE': (is_history_tuple, to_history_tuple, history_tuple_to_str),
     'XONSH_LOGIN': (is_bool, to_bool, bool_to_str),
     'XONSH_PROC_FREQUENCY': (is_float, float, str),
@@ -236,7 +234,6 @@ def xonsh_config_dir(env):
     return xcd
 
 
-@default_value
 def xonshconfig(env):
     """Ensures and returns the $XONSHCONFIG"""
     xcd = env.get('XONSH_CONFIG_DIR')
@@ -249,13 +246,18 @@ def default_xonshrc(env):
     """Creates a new instance of the default xonshrc tuple."""
     xcdrc = os.path.join(xonsh_config_dir(env), 'rc.xsh')
     if ON_WINDOWS:
-        dxrc = (xonshconfig(env),
-                os.path.join(os_environ['ALLUSERSPROFILE'],
+        dxrc = (os.path.join(os_environ['ALLUSERSPROFILE'],
                              'xonsh', 'xonshrc'),
                 xcdrc,
                 os.path.expanduser('~/.xonshrc'))
     else:
-        dxrc = (xonshconfig(env), '/etc/xonshrc', xcdrc, os.path.expanduser('~/.xonshrc'))
+        dxrc = ('/etc/xonshrc', xcdrc, os.path.expanduser('~/.xonshrc'))
+    # Check if old config file exists and issue warning
+    old_config_filename = xonshconfig(env)
+    if os.path.isfile(old_config_filename):
+        print('WARNING! old style configuration (' +
+              old_config_filename + ') is no longer supported. ' +
+              'Please migrate to xonshrc.')
     return dxrc
 
 
@@ -274,7 +276,7 @@ def DEFAULT_VALUES():
         'AUTO_CD': False,
         'AUTO_PUSHD': False,
         'AUTO_SUGGEST': True,
-        'AUTO_SUGGEST_IN_COMPLETIONS': True,
+        'AUTO_SUGGEST_IN_COMPLETIONS': False,
         'BASH_COMPLETIONS': BASH_COMPLETIONS_DEFAULT,
         'CASE_SENSITIVE_COMPLETIONS': ON_LINUX,
         'CDPATH': (),
@@ -304,7 +306,6 @@ def DEFAULT_VALUES():
         'LC_TIME': locale.setlocale(locale.LC_TIME),
         'LC_MONETARY': locale.setlocale(locale.LC_MONETARY),
         'LC_NUMERIC': locale.setlocale(locale.LC_NUMERIC),
-        'LOADED_CONFIG': False,
         'LOADED_RC_FILES': (),
         'MOUSE_SUPPORT': False,
         'MULTILINE_PROMPT': '.',
@@ -312,6 +313,7 @@ def DEFAULT_VALUES():
         'PATHEXT': ['.COM', '.EXE', '.BAT', '.CMD'] if ON_WINDOWS else [],
         'PRETTY_PRINT_RESULTS': True,
         'PROMPT': prompt.default_prompt(),
+        'PROMPT_TOOLKIT_COLOR_DEPTH': '',
         'PUSHD_MINUS': False,
         'PUSHD_SILENT': False,
         'RAISE_SUBPROC_ERROR': False,
@@ -334,7 +336,6 @@ def DEFAULT_VALUES():
         'XDG_CONFIG_HOME': os.path.expanduser(os.path.join('~', '.config')),
         'XDG_DATA_HOME': os.path.expanduser(os.path.join('~', '.local',
                                                          'share')),
-        'XONSHCONFIG': xonshconfig,
         'XONSHRC': default_xonshrc,
         'XONSH_APPEND_NEWLINE': xonsh_append_newline,
         'XONSH_AUTOPAIR': False,
@@ -348,6 +349,7 @@ def DEFAULT_VALUES():
         'XONSH_ENCODING_ERRORS': 'surrogateescape',
         'XONSH_HISTORY_BACKEND': 'json',
         'XONSH_HISTORY_FILE': os.path.expanduser('~/.xonsh_history.json'),
+        'XONSH_HISTORY_MATCH_ANYWHERE': False,
         'XONSH_HISTORY_SIZE': (8128, 'commands'),
         'XONSH_LOGIN': False,
         'XONSH_PROC_FREQUENCY': 1e-4,
@@ -407,7 +409,7 @@ def DEFAULT_DOCS():
     'AUTO_SUGGEST_IN_COMPLETIONS': VarDocs(
         'Places the auto-suggest result as the first option in the completions. '
         'This enables you to tab complete the auto-suggestion.'
-        ),
+    ),
     'BASH_COMPLETIONS': VarDocs(
         'This is a list (or tuple) of strings that specifies where the '
         '``bash_completion`` script may be found. '
@@ -448,6 +450,8 @@ def DEFAULT_DOCS():
         '  single column while typing.\n'
         "- If ``$COMPLETIONS_DISPLAY`` is ``multi`` or ``true``, display completions\n"
         "  in multiple columns while typing.\n\n"
+        "- If ``$COMPLETIONS_DISPLAY`` is ``readline``, display completions\n"
+        "  will emulate the behavior of readline.\n\n"
         'These option values are not case- or type-sensitive, so e.g.'
         "writing ``$COMPLETIONS_DISPLAY = None`` "
         "and ``$COMPLETIONS_DISPLAY = 'none'`` are equivalent. Only usable with "
@@ -481,9 +485,8 @@ def DEFAULT_DOCS():
     'FOREIGN_ALIASES_OVERRIDE': VarDocs(
         'Whether or not foreign aliases should override xonsh aliases '
         'with the same name. Note that setting of this must happen in the '
-        'static configuration file '
-        "``$XONSH_CONFIG_DIR/config.json`` in the 'env' section and not in "
-        '``.xonshrc`` as loading of foreign aliases happens before'
+        'environment that xonsh was started from. '
+        "It cannot be set in the ``.xonshrc`` as loading of foreign aliases happens before"
         '``.xonshrc`` is parsed', configurable=True),
     'PROMPT_FIELDS': VarDocs(
         'Dictionary containing variables to be used when formatting $PROMPT '
@@ -515,9 +518,6 @@ def DEFAULT_DOCS():
         'generally replaced by their bright counter parts.',
         configurable=ON_WINDOWS),
     'LANG': VarDocs('Fallback locale setting for systems where it matters'),
-    'LOADED_CONFIG': VarDocs(
-        'Whether or not the xonsh config file was loaded',
-        configurable=False),
     'LOADED_RC_FILES': VarDocs(
         'Whether or not any of the xonsh run control files were loaded at '
         'startup. This is a sequence of bools in Python that is converted '
@@ -548,6 +548,10 @@ def DEFAULT_DOCS():
         'http://xon.sh/tutorial.html#customizing-the-prompt. '
         'This value is never inherited from parent processes.',
         default='``xonsh.environ.DEFAULT_PROMPT``'),
+    'PROMPT_TOOLKIT_COLOR_DEPTH': VarDocs(
+        'The color depth used by prompt toolkit 2. Possible values are: '
+        '``DEPTH_1_BIT``, ``DEPTH_4_BIT``, ``DEPTH_8_BIT``, ``DEPTH_24_BIT`` '
+        'colors. Default is an empty string which means that prompt toolkit decide.'),
     'PUSHD_MINUS': VarDocs(
         'Flag for directory pushing functionality. False is the normal '
         'behavior.'),
@@ -620,7 +624,7 @@ def DEFAULT_DOCS():
         'Completions display is evaluated and presented whenever a key is '
         'pressed. This avoids the need to press TAB, except to cycle through '
         'the possibilities. This currently only affects the prompt-toolkit shell.'
-        ),
+    ),
     'UPDATE_OS_ENVIRON': VarDocs(
         "If True ``os_environ`` will always be updated "
         "when the xonsh environment changes. The environment can be reset to "
@@ -650,10 +654,6 @@ def DEFAULT_DOCS():
     'XDG_DATA_HOME': VarDocs(
         'Open desktop standard data home dir. This is the same default as '
         'used in the standard.', default="``~/.local/share``"),
-    'XONSHCONFIG': VarDocs(
-        'The location of the static xonsh configuration file, if it exists. '
-        'This is in JSON format.', configurable=False,
-        default="``$XONSH_CONFIG_DIR/config.json``"),
     'XONSHRC': VarDocs(
         'A list of the locations of run control files, if they exist.  User '
         'defined run control file will supersede values set in system-wide '
@@ -723,6 +723,10 @@ def DEFAULT_DOCS():
     'XONSH_HISTORY_FILE': VarDocs(
         'Location of history file (deprecated).',
         configurable=False, default="``~/.xonsh_history``"),
+    'XONSH_HISTORY_MATCH_ANYWHERE': VarDocs(
+        "When searching history from a partial string (by pressing up arrow), "
+        "match command history anywhere in a given line (not just the start)",
+        default="False"),
     'XONSH_HISTORY_SIZE': VarDocs(
         'Value and units tuple that sets the size of history after garbage '
         'collection. Canonical units are:\n\n'
@@ -823,12 +827,6 @@ class Env(cabc.MutableMapping):
             self._d['PATH'] = list(PATH_DEFAULT)
         self._detyped = None
 
-    @property
-    def arg_regex(self):
-        if self._arg_regex is None:
-            self._arg_regex = re.compile(r'ARG(\d+)')
-        return self._arg_regex
-
     @staticmethod
     def detypeable(val):
         return not (callable(val) or isinstance(val, cabc.MutableMapping))
@@ -926,13 +924,20 @@ class Env(cabc.MutableMapping):
             old[k] = self.get(k, NotImplemented)
             self[k] = v
 
-        yield self
-        # restore the values
-        for k, v in old.items():
-            if v is NotImplemented:
-                del self[k]
-            else:
-                self[k] = v
+        exception = None
+        try:
+            yield self
+        except Exception as e:
+            exception = e
+        finally:
+            # restore the values
+            for k, v in old.items():
+                if v is NotImplemented:
+                    del self[k]
+                else:
+                    self[k] = v
+            if exception is not None:
+                raise exception from None
 
     #
     # Mutable mapping interface
@@ -947,14 +952,6 @@ class Env(cabc.MutableMapping):
             return self['PROMPT_FIELDS']
         if key is Ellipsis:
             return self
-        m = self.arg_regex.match(key)
-        if (m is not None) and (key not in self._d) and ('ARGS' in self._d):
-            args = self._d['ARGS']
-            ix = int(m.group(1))
-            if ix >= len(args):
-                e = "Not enough arguments given to access ARG{0}."
-                raise KeyError(e.format(ix))
-            val = self._d['ARGS'][ix]
         elif key in self._d:
             val = self._d[key]
         elif key in self._defaults:
@@ -1059,54 +1056,6 @@ BASE_ENV = LazyObject(lambda: {
 }, globals(), 'BASE_ENV')
 
 
-def load_static_config(ctx, config=None):
-    """Loads a static configuration file from a given context, rather than the
-    current environment.  Optionally may pass in configuration file name.
-    """
-    env = {}
-    env['XDG_CONFIG_HOME'] = ctx.get('XDG_CONFIG_HOME',
-                                     DEFAULT_VALUES['XDG_CONFIG_HOME'])
-    env['XONSH_CONFIG_DIR'] = ctx['XONSH_CONFIG_DIR'] if 'XONSH_CONFIG_DIR' in ctx \
-        else xonsh_config_dir(env)
-    if config is not None:
-        env['XONSHCONFIG'] = ctx['XONSHCONFIG'] = config
-    elif 'XONSHCONFIG' in ctx:
-        config = env['XONSHCONFIG'] = ctx['XONSHCONFIG']
-    else:
-        # don't set in ctx in order to maintain default
-        config = env['XONSHCONFIG'] = xonshconfig(env)
-    if os.path.isfile(config):
-        # Note that an Env instance at __xonsh_env__ has not been started yet,
-        # per se, so we have to use os_environ
-        encoding = os_environ.get('XONSH_ENCODING',
-                                  DEFAULT_VALUES.get('XONSH_ENCODING', 'utf8'))
-        errors = os_environ.get('XONSH_ENCODING_ERRORS',
-                                DEFAULT_VALUES.get('XONSH_ENCODING_ERRORS',
-                                                   'surrogateescape'))
-        with open(config, 'r', encoding=encoding, errors=errors) as f:
-            try:
-                conf = json.load(f)
-                assert isinstance(conf, cabc.Mapping)
-                ctx['LOADED_CONFIG'] = True
-            except Exception as e:
-                conf = {}
-                ctx['LOADED_CONFIG'] = False
-                print_exception()
-                # JSONDecodeError was added in Python v3.5
-                jerr = json.JSONDecodeError \
-                    if hasattr(json, 'JSONDecodeError') else ValueError
-                if isinstance(e, jerr):
-                    msg = 'Xonsh config file is not valid JSON.'
-                else:
-                    msg = 'Could not load xonsh config.'
-                print(msg, file=sys.stderr)
-    else:
-        conf = {}
-        ctx['LOADED_CONFIG'] = False
-    builtins.__xonsh_config__ = conf
-    return conf
-
-
 def xonshrc_context(rcfiles=None, execer=None, ctx=None, env=None, login=True):
     """Attempts to read in all xonshrc files and return the context."""
     loaded = env['LOADED_RC_FILES'] = []
@@ -1119,12 +1068,8 @@ def xonshrc_context(rcfiles=None, execer=None, ctx=None, env=None, login=True):
             loaded.append(False)
             continue
         _, ext = os.path.splitext(rcfile)
-        if ext == '.json':
-            status = static_config_run_control(rcfile, ctx, env, execer=execer,
-                                               login=login)
-        else:
-            status = xonsh_script_run_control(rcfile, ctx, env, execer=execer,
-                                              login=login)
+        status = xonsh_script_run_control(rcfile, ctx, env, execer=execer,
+                                          login=login)
         loaded.append(status)
     return ctx
 
@@ -1152,41 +1097,6 @@ def foreign_env_fixes(ctx):
         del ctx['PROMPT']
 
 
-def static_config_run_control(filename, ctx, env, execer=None, login=True):
-    """Loads a static config file and applies it as a run control."""
-    if not login:
-        return
-    conf = load_static_config(env, config=filename)
-    # load foreign shells
-    foreign_env = load_foreign_envs(shells=conf.get('foreign_shells', ()),
-                                    issue_warning=False)
-    if ON_WINDOWS:
-        windows_foreign_env_fixes(foreign_env)
-    foreign_env_fixes(foreign_env)
-    env.update(foreign_env)
-    aliases = builtins.aliases
-    foreign_aliases = load_foreign_aliases(config=filename, issue_warning=True)
-    for k, v in foreign_aliases.items():
-        if k in aliases:
-            msg = ('Skipping application of {0!r} alias from foreign shell '
-                   '(loaded from {1!r}) since it shares a name with an '
-                   'existing xonsh alias.')
-            print(msg.format(k, filename))
-        else:
-            aliases[k] = v
-    # load xontribs
-    names = conf.get('xontribs', ())
-    for name in names:
-        update_context(name, ctx=ctx)
-    if getattr(update_context, 'bad_imports', None):
-        prompt_xontrib_install(update_context.bad_imports)
-        del update_context.bad_imports
-    # Do static config environment last, to allow user to override any of
-    # our environment choices
-    env.update(conf.get('env', ()))
-    return True
-
-
 def xonsh_script_run_control(filename, ctx, env, execer=None, login=True):
     """Loads a xonsh file and applies it as a run control."""
     if execer is None:
@@ -1225,3 +1135,14 @@ def default_env(env=None):
     if env is not None:
         ctx.update(env)
     return ctx
+
+
+def make_args_env(args=None):
+    """Makes a dictionary containing the $ARGS and $ARG<N> environment
+    variables. If the supplied ARGS is None, then sys.argv is used.
+    """
+    if args is None:
+        args = sys.argv
+    env = {'ARG' + str(i): arg for i, arg in enumerate(args)}
+    env['ARGS'] = list(args)  # make a copy so we don't interfere with original variable
+    return env
