@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 """Implements JSON version of xonsh history backend."""
 import os
+import sys
 import time
+import json
 import builtins
 import collections
 import threading
@@ -62,9 +64,9 @@ def _xhj_gc_bytes_to_rmfiles(hsize, files):
     return rmfiles
 
 
-def _xhj_get_history_files(sort=True, reverse=False):
+def _xhj_get_history_files(sort=True, newest_first=False):
     """Find and return the history files. Optionally sort files by
-        modify time.
+    modify time.
     """
     data_dir = builtins.__xonsh_env__.get('XONSH_DATA_DIR')
     data_dir = xt.expanduser_abs_path(data_dir)
@@ -76,7 +78,7 @@ def _xhj_get_history_files(sort=True, reverse=False):
         if builtins.__xonsh_env__.get('XONSH_DEBUG'):
             xt.print_exception("Could not collect xonsh history files.")
     if sort:
-        files.sort(key=lambda x: os.path.getmtime(x), reverse=reverse)
+        files.sort(key=lambda x: os.path.getmtime(x), reverse=newest_first)
     return files
 
 
@@ -338,7 +340,10 @@ class JsonHistory(History):
         Parameters
         ----------
         cmd : dict
-            Command dictionary that should be added to the ordered history.
+            This dict contains information about the command that is to be
+            added to the history list. It should contain the keys ``inp``,
+            ``rtn`` and ``ts``. These key names mirror the same names defined
+            as instance variables in the ``HistoryEntry`` class.
 
         Returns
         -------
@@ -360,7 +365,7 @@ class JsonHistory(History):
         ----------
         at_exit : bool, optional
             Whether the JsonHistoryFlusher should act as a thread in the
-            background, or execute immeadiately and block.
+            background, or execute immediately and block.
 
         Returns
         -------
@@ -374,26 +379,39 @@ class JsonHistory(History):
         self.buffer.clear()
         return hf
 
-    def items(self):
+    def items(self, newest_first=False):
         """Display history items of current session."""
-        for item, tss in zip(self.inps, self.tss):
+        if newest_first:
+            items = zip(reversed(self.inps), reversed(self.tss))
+        else:
+            items = zip(self.inps, self.tss)
+        for item, tss in items:
             yield {'inp': item.rstrip(), 'ts': tss[0]}
 
-    def all_items(self, **kwargs):
+    def all_items(self, newest_first=False, **kwargs):
         """
         Returns all history as found in XONSH_DATA_DIR.
 
-        yeild format: {'inp': cmd, 'rtn': 0, ...}
+        yield format: {'inp': cmd, 'rtn': 0, ...}
         """
         while self.gc and self.gc.is_alive():
             time.sleep(0.011)  # gc sleeps for 0.01 secs, sleep a beat longer
-        for f in _xhj_get_history_files():
+        for f in _xhj_get_history_files(newest_first=newest_first):
             try:
                 json_file = xlj.LazyJSON(f, reopen=False)
             except ValueError:
                 # Invalid json file
                 continue
-            commands = json_file.load()['cmds']
+            try:
+                commands = json_file.load()['cmds']
+            except json.decoder.JSONDecodeError:
+                # file is corrupted somehow
+                if builtins.__xonsh_env__.get('XONSH_DEBUG') > 0:
+                    msg = 'xonsh history file {0!r} is not valid JSON'
+                    print(msg.format(f), file=sys.stderr)
+                continue
+            if newest_first:
+                commands = reversed(commands)
             for c in commands:
                 yield {'inp': c['inp'].rstrip(), 'ts': c['ts'][0]}
         # all items should also include session items
