@@ -24,7 +24,8 @@ from xonsh.xonfig import print_welcome_screen
 from xonsh.lazyimps import pygments, pyghooks
 from xonsh.imphooks import install_import_hooks
 from xonsh.events import events
-from xonsh.environ import xonshrc_context
+from xonsh.environ import xonshrc_context, make_args_env
+from xonsh.xontribs import xontribs_load
 
 
 events.transmogrify('on_post_init', 'LoadEvent')
@@ -167,7 +168,7 @@ def parser():
                         'Possible options: readline, prompt_toolkit, random. '
                         'Warning! If set this overrides $SHELL_TYPE variable.',
                    dest='shell_type',
-                   choices=('readline', 'prompt_toolkit', 'best', 'random'),
+                   choices=tuple(Shell.shell_type_aliases.keys()),
                    default=None)
     p.add_argument('--timings',
                    help='Prints timing information before the prompt is shown. '
@@ -361,7 +362,6 @@ def main_xonsh(args):
             env['XONSH_INTERACTIVE'] = True
             ignore_sigtstp()
             if (env['XONSH_INTERACTIVE'] and
-                    not env['LOADED_CONFIG'] and
                     not any(os.path.isfile(i) for i in env['XONSHRC'])):
                 print_welcome_screen()
             events.on_pre_cmdloop.fire()
@@ -377,7 +377,7 @@ def main_xonsh(args):
             path = os.path.abspath(os.path.expanduser(args.file))
             if os.path.isfile(path):
                 sys.argv = [args.file] + args.args
-                env['ARGS'] = sys.argv[:]  # $ARGS is not sys.argv
+                env.update(make_args_env())  # $ARGS is not sys.argv
                 env['XONSH_SOURCE'] = path
                 shell.ctx.update({'__file__': args.file, '__name__': '__main__'})
                 run_script_with_cache(args.file, shell.execer, glb=shell.ctx,
@@ -413,7 +413,8 @@ def main_context(argv=None):
     postmain(args)
 
 
-def setup(ctx=None, shell_type='none', env=(('RAISE_SUBPROC_ERROR', True),)):
+def setup(ctx=None, shell_type='none', env=(('RAISE_SUBPROC_ERROR', True),),
+          aliases=(), xontribs=(), threadable_predictors=()):
     """Starts up a new xonsh shell. Calling this in function in another
     packages __init__.py will allow xonsh to be fully used in the
     package in headless or headed mode. This function is primarily indended to
@@ -430,6 +431,12 @@ def setup(ctx=None, shell_type='none', env=(('RAISE_SUBPROC_ERROR', True),)):
     env : dict-like, optional
         Environment to update the current environment with after the shell
         has been initialized.
+    aliases : dict-like, optional
+        Aliases to add after the shell has been initialized.
+    xontribs : iterable of str, optional
+        Xontrib names to load.
+    threadable_predictors : dict-like, optional
+        Threadable predictors to start up with. These overide the defaults.
     """
     ctx = {} if ctx is None else ctx
     # setup xonsh ctx and execer
@@ -437,6 +444,11 @@ def setup(ctx=None, shell_type='none', env=(('RAISE_SUBPROC_ERROR', True),)):
     builtins.__xonsh_execer__ = Execer(xonsh_ctx=ctx)
     builtins.__xonsh_shell__ = Shell(builtins.__xonsh_execer__,
                                      ctx=ctx,
-                                     shell_type='none')
+                                     shell_type=shell_type)
     builtins.__xonsh_env__.update(env)
     install_import_hooks()
+    builtins.aliases.update(aliases)
+    if xontribs:
+        xontribs_load(xontribs)
+    tp = builtins.__xonsh_commands_cache__.threadable_predictors
+    tp.update(threadable_predictors)
