@@ -47,6 +47,8 @@ ON_WINDOWS = LazyBool(lambda: platform.system() == 'Windows',
 """``True`` if executed on a native Windows platform, else ``False``. """
 ON_CYGWIN = LazyBool(lambda: sys.platform == 'cygwin', globals(), 'ON_CYGWIN')
 """``True`` if executed on a Cygwin Windows platform, else ``False``. """
+ON_MSYS = LazyBool(lambda: sys.platform == 'msys', globals(), 'ON_MSYS')
+"""``True`` if executed on a MSYS Windows platform, else ``False``. """
 ON_POSIX = LazyBool(lambda: (os.name == 'posix'), globals(), 'ON_POSIX')
 """``True`` if executed on a POSIX-compliant platform, else ``False``. """
 ON_FREEBSD = LazyBool(lambda: (sys.platform.startswith('freebsd')),
@@ -112,16 +114,25 @@ def pygments_version():
     return v
 
 
+@functools.lru_cache(1)
+def pygments_version_info():
+    """ Returns `pygments`'s version as tuple of integers. """
+    if HAS_PYGMENTS:
+        return tuple(int(x) for x in pygments_version().strip('<>+-=.').split('.'))
+    else:
+        return None
+
+
 @functools.lru_cache(1)
 def has_prompt_toolkit():
-    """ Tests if the `prompt_toolkit` is available. """
+    """Tests if the `prompt_toolkit` is available."""
     spec = importlib.util.find_spec('prompt_toolkit')
     return (spec is not None)
 
 
 @functools.lru_cache(1)
 def ptk_version():
-    """ Returns `prompt_toolkit.__version__` if available, else ``None``. """
+    """Returns `prompt_toolkit.__version__` if available, else ``None``."""
     if has_prompt_toolkit():
         import prompt_toolkit
         return getattr(prompt_toolkit, '__version__', '<0.57')
@@ -139,11 +150,38 @@ def ptk_version_info():
 
 
 @functools.lru_cache(1)
-def ptk_version_is_supported():
+def ptk_above_min_supported():
     minimum_required_ptk_version = (1, 0)
     return ptk_version_info()[:2] >= minimum_required_ptk_version
 
 
+@functools.lru_cache(1)
+def ptk_shell_type():
+    """Returns the prompt_toolkit shell type based on the installed version."""
+    if ptk_version_info()[:2] < (2, 0):
+        return 'prompt_toolkit1'
+    else:
+        return 'prompt_toolkit2'
+
+
+@functools.lru_cache(1)
+def win_ansi_support():
+    if ON_WINDOWS:
+        try:
+            from prompt_toolkit.utils import is_windows_vt100_supported, is_conemu_ansi
+        except ImportError:
+            return False
+        return is_conemu_ansi() or is_windows_vt100_supported()
+    else:
+        return False
+
+
+@functools.lru_cache(1)
+def ptk_below_max_supported():
+    ptk_max_version_cutoff = (2, 0)
+    return ptk_version_info()[:2] < ptk_max_version_cutoff
+
+
 @functools.lru_cache(1)
 def best_shell_type():
     if ON_WINDOWS or has_prompt_toolkit():
@@ -173,7 +211,7 @@ def pathsplit(p):
     without a drive.
     """
     n = len(p)
-    while n and p[n-1] not in seps:
+    while n and p[n - 1] not in seps:
         n -= 1
     pre = p[:n]
     pre = pre.rstrip(seps) or pre
@@ -266,14 +304,14 @@ if PYTHON_VERSION_INFO < (3, 5, 0):
                 return self.__path__.is_dir()
             else:
                 return not self.__path__.is_symlink() \
-                       and self.__path__.is_dir()
+                    and self.__path__.is_dir()
 
         def is_file(self, *, follow_symlinks=True):
             if follow_symlinks:
                 return self.__path__.is_file()
             else:
                 return not self.__path__.is_symlink() \
-                       and self.__path__.is_file()
+                    and self.__path__.is_file()
 
         def stat(self, *, follow_symlinks=True):
             return os.stat(self.path, follow_symlinks=follow_symlinks)
@@ -440,7 +478,7 @@ def BASH_COMPLETIONS_DEFAULT():
     """A possibly empty tuple with default paths to Bash completions known for
     the current platform.
     """
-    if ON_LINUX or ON_CYGWIN:
+    if ON_LINUX or ON_CYGWIN or ON_MSYS:
         bcd = ('/usr/share/bash-completion/bash_completion', )
     elif ON_DARWIN:
         bcd = ('/usr/local/share/bash-completion/bash_completion',  # v2.x
@@ -458,7 +496,7 @@ def BASH_COMPLETIONS_DEFAULT():
 
 @lazyobject
 def PATH_DEFAULT():
-    if ON_LINUX or ON_CYGWIN:
+    if ON_LINUX or ON_CYGWIN or ON_MSYS:
         if linux_distro() == 'arch':
             pd = ('/usr/local/sbin',
                   '/usr/local/bin', '/usr/bin', '/usr/bin/site_perl',
@@ -485,10 +523,14 @@ def PATH_DEFAULT():
 @lazyobject
 def LIBC():
     """The platform dependent libc implementation."""
+    global ctypes
     if ON_DARWIN:
+        import ctypes.util
         libc = ctypes.CDLL(ctypes.util.find_library("c"))
     elif ON_CYGWIN:
         libc = ctypes.CDLL('cygwin1.dll')
+    elif ON_MSYS:
+        libc = ctypes.CDLL('msys-2.0.dll')
     elif ON_BSD:
         try:
             libc = ctypes.CDLL('libc.so')
