@@ -23,8 +23,8 @@ import threading
 import subprocess
 import collections.abc as cabc
 
-from xonsh.platform import (ON_WINDOWS, ON_POSIX, CAN_RESIZE_WINDOW,
-                            LFLAG, CC)
+from xonsh.platform import (ON_WINDOWS, ON_POSIX, ON_MSYS, ON_CYGWIN,
+                            CAN_RESIZE_WINDOW, LFLAG, CC)
 from xonsh.tools import (redirect_stdout, redirect_stderr, print_exception,
                          XonshCalledProcessError, findfirst, on_main_thread,
                          XonshError, format_std_prepost)
@@ -96,9 +96,9 @@ class QueueReader:
         """Returns whether or not the queue is fully read and the reader is
         closed.
         """
-        return (self.closed
-                and (self.thread is None or not self.thread.is_alive())
-                and self.queue.empty())
+        return (self.closed and
+                (self.thread is None or not self.thread.is_alive()) and
+                self.queue.empty())
 
     def read_queue(self):
         """Reads a single chunk from the queue. This is blocking if
@@ -272,7 +272,7 @@ def _expand_console_buffer(cols, max_offset, expandsize, orig_posize, fd):
     # expand it so that we can read from it successfully.
     if cols == 0:
         return orig_posize[-1], max_offset, orig_posize
-    rows = ((max_offset + expandsize)//cols) + 1
+    rows = ((max_offset + expandsize) // cols) + 1
     winutils.set_console_screen_buffer_size(cols, rows, fd=fd)
     orig_posize = orig_posize[:3] + (rows,)
     max_offset = (rows - 1) * cols
@@ -328,7 +328,7 @@ def populate_console(reader, fd, buffer, chunksize, queue, expandsize=None):
     x, y, cols, rows = posize = winutils.get_position_size(fd)
     pre_x = pre_y = -1
     orig_posize = posize
-    offset = (cols*y) + x
+    offset = (cols * y) + x
     max_offset = (rows - 1) * cols
     # I believe that there is a bug in PTK that if we reset the
     # cursor position, the cursor on the next prompt is accidentally on
@@ -340,7 +340,7 @@ def populate_console(reader, fd, buffer, chunksize, queue, expandsize=None):
     #     winutils.set_console_cursor_position(x, y, fd=fd)
     while True:
         posize = winutils.get_position_size(fd)
-        offset = (cols*y) + x
+        offset = (cols * y) + x
         if ((posize[1], posize[0]) <= (y, x) and posize[2:] == (cols, rows)) or \
                 (pre_x == x and pre_y == y):
             # already at or ahead of the current cursor position.
@@ -382,11 +382,11 @@ def populate_console(reader, fd, buffer, chunksize, queue, expandsize=None):
             time.sleep(reader.timeout)
             continue
         cur_x, cur_y = posize[0], posize[1]
-        cur_offset = (cols*cur_y) + cur_x
-        beg_offset = (cols*y) + x
+        cur_offset = (cols * cur_y) + cur_x
+        beg_offset = (cols * y) + x
         end_offset = beg_offset + nread
         if end_offset > cur_offset and cur_offset != max_offset:
-            buf = buf[:cur_offset-end_offset]
+            buf = buf[:cur_offset - end_offset]
         # convert to lines
         xshift = cols - x
         yshift = (nread // cols) + (1 if nread % cols > 0 else 0)
@@ -549,7 +549,7 @@ class PopenThread(threading.Thread):
                                                        self._signal_winch)
         # start up process
         if ON_WINDOWS and stdout is not None:
-            os.set_handle_inheritable(stdout.fileno(), False)
+            os.set_inheritable(stdout.fileno(), False)
 
         try:
             self.proc = proc = subprocess.Popen(*args,
@@ -895,7 +895,7 @@ class PopenThread(threading.Thread):
         if s is None:
             rtn = self.returncode
             if rtn is not None and rtn != 0:
-                s = (-1*rtn, rtn < 0 if ON_WINDOWS else os.WCOREDUMP(rtn))
+                s = (-1 * rtn, rtn < 0 if ON_WINDOWS else os.WCOREDUMP(rtn))
         return s
 
     @signal.setter
@@ -1157,36 +1157,44 @@ def parse_proxy_return(r, stdout, stderr):
     return cmd_result
 
 
-def proxy_zero(f, args, stdin, stdout, stderr, spec):
+def proxy_zero(f, args, stdin, stdout, stderr, spec, stack):
     """Calls a proxy function which takes no parameters."""
     return f()
 
 
-def proxy_one(f, args, stdin, stdout, stderr, spec):
+def proxy_one(f, args, stdin, stdout, stderr, spec, stack):
     """Calls a proxy function which takes one parameter: args"""
     return f(args)
 
 
-def proxy_two(f, args, stdin, stdout, stderr, spec):
+def proxy_two(f, args, stdin, stdout, stderr, spec, stack):
     """Calls a proxy function which takes two parameter: args and stdin."""
     return f(args, stdin)
 
 
-def proxy_three(f, args, stdin, stdout, stderr, spec):
+def proxy_three(f, args, stdin, stdout, stderr, spec, stack):
     """Calls a proxy function which takes three parameter: args, stdin, stdout.
     """
     return f(args, stdin, stdout)
 
 
-def proxy_four(f, args, stdin, stdout, stderr, spec):
+def proxy_four(f, args, stdin, stdout, stderr, spec, stack):
     """Calls a proxy function which takes four parameter: args, stdin, stdout,
     and stderr.
     """
     return f(args, stdin, stdout, stderr)
 
 
-PROXIES = (proxy_zero, proxy_one, proxy_two, proxy_three, proxy_four)
-PROXY_KWARG_NAMES = frozenset(['args', 'stdin', 'stdout', 'stderr', 'spec'])
+def proxy_five(f, args, stdin, stdout, stderr, spec, stack):
+    """Calls a proxy function which takes four parameter: args, stdin, stdout,
+    stderr, and spec.
+    """
+    return f(args, stdin, stdout, stderr, spec)
+
+
+PROXIES = (proxy_zero, proxy_one, proxy_two, proxy_three, proxy_four, proxy_five)
+PROXY_KWARG_NAMES = frozenset(['args', 'stdin', 'stdout', 'stderr', 'spec',
+                               'stack'])
 
 
 def partial_proxy(f):
@@ -1198,13 +1206,13 @@ def partial_proxy(f):
             numargs += 1
         elif name in PROXY_KWARG_NAMES and param.kind == param.KEYWORD_ONLY:
             numargs += 1
-    if numargs < 5:
+    if numargs < 6:
         return functools.partial(PROXIES[numargs], f)
-    elif numargs == 5:
+    elif numargs == 6:
         # don't need to partial.
         return f
     else:
-        e = 'Expected proxy with 5 or fewer arguments for {}, not {}'
+        e = 'Expected proxy with 6 or fewer arguments for {}, not {}'
         raise XonshError(e.format(', '.join(PROXY_KWARG_NAMES), numargs))
 
 
@@ -1345,15 +1353,15 @@ class ProcProxyThread(threading.Thread):
         # run the function itself
         try:
             with STDOUT_DISPATCHER.register(sp_stdout), \
-                 STDERR_DISPATCHER.register(sp_stderr), \
-                 redirect_stdout(STDOUT_DISPATCHER), \
-                 redirect_stderr(STDERR_DISPATCHER):
-                r = self.f(self.args, sp_stdin, sp_stdout, sp_stderr, spec)
+            STDERR_DISPATCHER.register(sp_stderr), \
+            redirect_stdout(STDOUT_DISPATCHER), \
+            redirect_stderr(STDERR_DISPATCHER):
+                r = self.f(self.args, sp_stdin, sp_stdout, sp_stderr, spec,
+                           spec.stack)
         except SystemExit as e:
             r = e.code if isinstance(e.code, int) else int(bool(e.code))
         except OSError as e:
-            status = still_writable(self.c2pwrite) and \
-                     still_writable(self.errwrite)
+            status = still_writable(self.c2pwrite) and still_writable(self.errwrite)
             if status:
                 # stdout and stderr are still writable, so error must
                 # come from function itself.
@@ -1629,7 +1637,7 @@ class ProcProxy(object):
         stderr = self._pick_buf(self.stderr, sys.stderr, enc, err)
         # run the actual function
         try:
-            r = self.f(self.args, stdin, stdout, stderr, spec)
+            r = self.f(self.args, stdin, stdout, stderr, spec, spec.stack)
         except Exception:
             print_exception()
             r = 1
@@ -1671,13 +1679,13 @@ def SIGNAL_MESSAGES():
         signal.SIGILL: 'Illegal instructions',
         signal.SIGTERM: 'Terminated',
         signal.SIGSEGV: 'Segmentation fault',
-        }
+    }
     if ON_POSIX:
         sm.update({
             signal.SIGQUIT: 'Quit',
             signal.SIGHUP: 'Hangup',
             signal.SIGKILL: 'Killed',
-            })
+        })
     return sm
 
 
@@ -1765,9 +1773,11 @@ class CommandPipeline:
                 self.starttime = time.time()
             try:
                 proc = spec.run(pipeline_group=pipeline_group)
-            except XonshError:
+            except Exception:
+                print_exception()
                 self._return_terminal()
-                raise
+                self.proc = None
+                return
             if proc.pid and pipeline_group is None and not spec.is_proxy and \
                     self.captured != 'object':
                 pipeline_group = proc.pid
@@ -1804,11 +1814,13 @@ class CommandPipeline:
         # get appropriate handles
         spec = self.spec
         proc = self.proc
+        if proc is None:
+            return
         timeout = builtins.__xonsh_env__.get('XONSH_PROC_FREQUENCY')
         # get the correct stdout
         stdout = proc.stdout
-        if ((stdout is None or spec.stdout is None or not safe_readable(stdout))
-                and spec.captured_stdout is not None):
+        if ((stdout is None or spec.stdout is None or not safe_readable(stdout)) and
+                spec.captured_stdout is not None):
             stdout = spec.captured_stdout
         if hasattr(stdout, 'buffer'):
             stdout = stdout.buffer
@@ -1834,11 +1846,11 @@ class CommandPipeline:
                     b = stdout.read()
                     s = self._decode_uninew(b, universal_newlines=True)
                     self.lines = s.splitlines(keepends=True)
-            raise StopIteration
+            return
         # get the correct stderr
         stderr = proc.stderr
-        if ((stderr is None or spec.stderr is None or not safe_readable(stderr))
-                and spec.captured_stderr is not None):
+        if ((stderr is None or spec.stderr is None or not safe_readable(stderr)) and
+                spec.captured_stderr is not None):
             stderr = spec.captured_stderr
         if hasattr(stderr, 'buffer'):
             stderr = stderr.buffer
@@ -2062,25 +2074,31 @@ class CommandPipeline:
         """Boolean for if all previous processes have completed. If there
         is only a single process in the pipeline, this returns False.
         """
+        any_running = False
         for s, p in zip(self.specs[:-1], self.procs[:-1]):
-            self._safe_close(p.stdin)
-            self._safe_close(s.stdin)
             if p.poll() is None:
-                return False
-            self._safe_close(p.stdout)
+                any_running = True
+                continue
+            self._safe_close(s.stdin)
             self._safe_close(s.stdout)
-            self._safe_close(p.stderr)
             self._safe_close(s.stderr)
-        return len(self) > 1
+            if p is None:
+                continue
+            self._safe_close(p.stdin)
+            self._safe_close(p.stdout)
+            self._safe_close(p.stderr)
+        return False if any_running else (len(self) > 1)
 
     def _close_prev_procs(self):
         """Closes all but the last proc's stdout."""
         for s, p in zip(self.specs[:-1], self.procs[:-1]):
             self._safe_close(s.stdin)
-            self._safe_close(p.stdin)
             self._safe_close(s.stdout)
-            self._safe_close(p.stdout)
             self._safe_close(s.stderr)
+            if p is None:
+                continue
+            self._safe_close(p.stdin)
+            self._safe_close(p.stdout)
             self._safe_close(p.stderr)
 
     def _close_proc(self):
@@ -2088,16 +2106,20 @@ class CommandPipeline:
         s = self.spec
         p = self.proc
         self._safe_close(s.stdin)
-        self._safe_close(p.stdin)
         self._safe_close(s.stdout)
-        self._safe_close(p.stdout)
         self._safe_close(s.stderr)
-        self._safe_close(p.stderr)
         self._safe_close(s.captured_stdout)
         self._safe_close(s.captured_stderr)
+        if p is None:
+            return
+        self._safe_close(p.stdin)
+        self._safe_close(p.stdout)
+        self._safe_close(p.stderr)
 
     def _set_input(self):
         """Sets the input variable."""
+        if self.proc is None:
+            return
         stdin = self.proc.stdin
         if stdin is None or isinstance(stdin, int) or stdin.closed or \
            not stdin.seekable() or not safe_readable(stdin):
@@ -2125,7 +2147,7 @@ class CommandPipeline:
         """Applies the results to the current history object."""
         hist = builtins.__xonsh_history__
         if hist is not None:
-            hist.last_cmd_rtn = self.proc.returncode
+            hist.last_cmd_rtn = 1 if self.proc is None else self.proc.returncode
 
     def _raise_subproc_error(self):
         """Raises a subprocess error, if we are supposed to."""
@@ -2135,8 +2157,12 @@ class CommandPipeline:
                 rtn is not None and
                 rtn > 0 and
                 builtins.__xonsh_env__.get('RAISE_SUBPROC_ERROR')):
-            raise subprocess.CalledProcessError(rtn, spec.cmd,
-                                                output=self.output)
+            try:
+                raise subprocess.CalledProcessError(rtn, spec.cmd,
+                                                    output=self.output)
+            finally:
+                # this is need to get a working terminal in interactive mode
+                self._return_terminal()
 
     #
     # Properties
@@ -2164,9 +2190,13 @@ class CommandPipeline:
 
     @property
     def output(self):
-        if self._output is None:
-            self._output = ''.join(self.lines)
-        return self._output
+        """Non-blocking, lazy access to output"""
+        if self.ended:
+            if self._output is None:
+                self._output = ''.join(self.lines)
+            return self._output
+        else:
+            return ''.join(self.lines)
 
     @property
     def out(self):
@@ -2189,6 +2219,8 @@ class CommandPipeline:
     def returncode(self):
         """Process return code, waits until command is completed."""
         self.end()
+        if self.proc is None:
+            return 1
         return self.proc.returncode
 
     rtn = returncode
@@ -2285,7 +2317,8 @@ def pause_call_resume(p, f, *args, **kwargs):
     args : remaining arguments
     kwargs : keyword arguments
     """
-    can_send_signal = hasattr(p, 'send_signal') and ON_POSIX
+    can_send_signal = (hasattr(p, 'send_signal') and ON_POSIX and
+                       not ON_MSYS and not ON_CYGWIN)
     if can_send_signal:
         p.send_signal(signal.SIGSTOP)
     try:
