@@ -10,16 +10,23 @@ from prompt_toolkit.shortcuts import print_tokens
 from prompt_toolkit.styles import PygmentsStyle, style_from_dict
 
 from xonsh.base_shell import BaseShell
-from xonsh.tools import print_exception
+from xonsh.tools import print_exception, carriage_return, ansicolors_to_ptk1_names
 from xonsh.ptk.completer import PromptToolkitCompleter
 from xonsh.ptk.history import PromptToolkitHistory
 from xonsh.ptk.key_bindings import load_xonsh_bindings
 from xonsh.ptk.shortcuts import Prompter
 from xonsh.events import events
 from xonsh.shell import transform_command
-from xonsh.platform import HAS_PYGMENTS
-from xonsh.style_tools import partial_color_tokenize, _TokenType, DEFAULT_STYLE_DICT
-from xonsh.lazyimps import pygments, pyghooks
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.style_tools import (
+    partial_color_tokenize,
+    _TokenType,
+    DEFAULT_STYLE_DICT as _DEFAULT_STYLE_DICT
+)
+from xonsh.lazyimps import pygments, pyghooks, winutils
+from xonsh.pygments_cache import get_all_styles
+from xonsh.lazyasd import LazyObject
+
 
 Token = _TokenType()
 
@@ -30,20 +37,31 @@ on_ptk_create(prompter: Prompter, history: PromptToolkitHistory, completer: Prom
 Fired after prompt toolkit has been initialized
 """)
 
+# Convert new ansicolor names to names
+# understood by PTK1
+DEFAULT_STYLE_DICT = LazyObject(
+    lambda: ansicolors_to_ptk1_names(_DEFAULT_STYLE_DICT),
+    globals(),
+    'DEFAULT_STYLE_DICT'
+)
+
 
 class PromptToolkitShell(BaseShell):
     """The xonsh shell."""
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
+        if ON_WINDOWS:
+            winutils.enable_virtual_terminal_processing()
+        self._first_prompt = True
         self.prompter = Prompter()
         self.history = PromptToolkitHistory()
-        self.pt_completer = PromptToolkitCompleter(self.completer, self.ctx)
+        self.pt_completer = PromptToolkitCompleter(self.completer, self.ctx, self)
         key_bindings_manager_args = {
             'enable_auto_suggest_bindings': True,
             'enable_search': True,
             'enable_abort_and_exit_bindings': True,
-            }
+        }
         self.key_bindings_manager = KeyBindingManager(**key_bindings_manager_args)
         load_xonsh_bindings(self.key_bindings_manager)
         # This assumes that PromptToolkitShell is a singleton
@@ -60,6 +78,7 @@ class PromptToolkitShell(BaseShell):
         kwarg flags whether the input should be stored in PTK's in-memory
         history.
         """
+        events.on_pre_prompt.fire()
         env = builtins.__xonsh_env__
         mouse_support = env.get('MOUSE_SUPPORT')
         if store_in_history:
@@ -70,6 +89,10 @@ class PromptToolkitShell(BaseShell):
         auto_suggest = auto_suggest if env.get('AUTO_SUGGEST') else None
         completions_display = env.get('COMPLETIONS_DISPLAY')
         multicolumn = (completions_display == 'multi')
+        complete_while_typing = env.get('UPDATE_COMPLETIONS_ON_KEYPRESS')
+        if complete_while_typing:
+            # PTK requires history search to be none when completing while typing
+            enable_history_search = False
         if HAS_PYGMENTS:
             self.styler.style_name = env.get('XONSH_COLOR_STYLE')
         completer = None if completions_display == 'none' else self.pt_completer
@@ -87,28 +110,29 @@ class PromptToolkitShell(BaseShell):
 
         with self.prompter:
             prompt_args = {
-                    'mouse_support': mouse_support,
-                    'auto_suggest': auto_suggest,
-                    'get_prompt_tokens': get_prompt_tokens,
-                    'get_rprompt_tokens': get_rprompt_tokens,
-                    'get_bottom_toolbar_tokens': get_bottom_toolbar_tokens,
-                    'completer': completer,
-                    'multiline': multiline,
-                    'get_continuation_tokens': self.continuation_tokens,
-                    'history': history,
-                    'enable_history_search': enable_history_search,
-                    'reserve_space_for_menu': 0,
-                    'key_bindings_registry': self.key_bindings_manager.registry,
-                    'display_completions_in_columns': multicolumn,
-                    }
+                'mouse_support': mouse_support,
+                'auto_suggest': auto_suggest,
+                'get_prompt_tokens': get_prompt_tokens,
+                'get_rprompt_tokens': get_rprompt_tokens,
+                'get_bottom_toolbar_tokens': get_bottom_toolbar_tokens,
+                'completer': completer,
+                'multiline': multiline,
+                'get_continuation_tokens': self.continuation_tokens,
+                'history': history,
+                'enable_history_search': enable_history_search,
+                'reserve_space_for_menu': 0,
+                'key_bindings_registry': self.key_bindings_manager.registry,
+                'display_completions_in_columns': multicolumn,
+                'complete_while_typing': complete_while_typing,
+            }
             if builtins.__xonsh_env__.get('COLOR_INPUT'):
                 if HAS_PYGMENTS:
                     prompt_args['lexer'] = PygmentsLexer(pyghooks.XonshLexer)
                     prompt_args['style'] = PygmentsStyle(pyghooks.xonsh_style_proxy(self.styler))
                 else:
                     prompt_args['style'] = style_from_dict(DEFAULT_STYLE_DICT)
-
             line = self.prompter.prompt(**prompt_args)
+            events.on_post_prompt.fire()
         return line
 
     def _push(self, line):
@@ -147,7 +171,7 @@ class PromptToolkitShell(BaseShell):
                 else:
                     line = self.precmd(line)
                     self.default(line)
-            except KeyboardInterrupt:
+            except (KeyboardInterrupt, SystemExit):
                 self.reset_buffer()
             except EOFError:
                 if builtins.__xonsh_env__.get("IGNOREEOF"):
@@ -163,6 +187,9 @@ class PromptToolkitShell(BaseShell):
         except Exception:  # pylint: disable=broad-except
             print_exception()
         toks = partial_color_tokenize(p)
+        if self._first_prompt:
+            carriage_return()
+            self._first_prompt = False
         self.settitle()
         return toks
 
@@ -206,11 +233,11 @@ class PromptToolkitShell(BaseShell):
         dots = builtins.__xonsh_env__.get('MULTILINE_PROMPT')
         dots = dots() if callable(dots) else dots
         if dots is None:
-            return [(Token, ' '*(width + 1))]
+            return [(Token, ' ' * (width + 1))]
         basetoks = self.format_color(dots)
         baselen = sum(len(t[1]) for t in basetoks)
         if baselen == 0:
-            return [(Token, ' '*(width + 1))]
+            return [(Token, ' ' * (width + 1))]
         toks = basetoks * (width // baselen)
         n = width % baselen
         count = 0
@@ -222,7 +249,7 @@ class PromptToolkitShell(BaseShell):
             elif newcount <= n:
                 toks.append(tok)
             else:
-                toks.append((tok[0], tok[1][:n-count]))
+                toks.append((tok[0], tok[1][:n - count]))
             count = newcount
             if n <= count:
                 break
@@ -267,7 +294,7 @@ class PromptToolkitShell(BaseShell):
         """Returns an iterable of all available style names."""
         if not HAS_PYGMENTS:
             return ['For other xonsh styles, please install pygments']
-        return pygments.styles.get_all_styles()
+        return get_all_styles()
 
     def color_style(self):
         """Returns the current color map."""
@@ -294,8 +321,8 @@ class PromptToolkitShell(BaseShell):
         # needs to be performed by the subprocess itself. This fix is important
         # when subprocesses don't properly restore the terminal attributes,
         # like Python in interactive mode. Also note that the sequences "\033M"
-        # and "\033E" seem to work too, but these are techinally VT100 codes.
-        # I used the more primitive ANSI sequence to maximize compatability.
+        # and "\033E" seem to work too, but these are technically VT100 codes.
+        # I used the more primitive ANSI sequence to maximize compatibility.
         # -scopatz 2017-01-28
         #   if not ON_POSIX:
         #       return
