@@ -8,32 +8,37 @@ import builtins
 from collections import ChainMap
 from collections.abc import MutableMapping
 
-# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-#           DO NOT MOVE
-# must come before pygments imports
-from xonsh.lazyasd import load_module_in_background
-load_module_in_background('pkg_resources', debug='XONSH_DEBUG',
-                          replacements={'pygments.plugin': 'pkg_resources'})
-# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-
 from pygments.lexer import inherit, bygroups, include
 from pygments.lexers.agile import PythonLexer
 from pygments.token import (Keyword, Name, Comment, String, Error, Number,
                             Operator, Generic, Whitespace, Token, Punctuation,
                             Text)
 from pygments.style import Style
-from pygments.styles import get_style_by_name
 import pygments.util
 
 from xonsh.commands_cache import CommandsCache
 from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
-from xonsh.tools import (ON_WINDOWS, intensify_colors_for_cmd_exe,
-                         expand_gray_colors_for_cmd_exe)
+from xonsh.tools import (
+    ON_WINDOWS,
+    intensify_colors_for_cmd_exe,
+    ansicolors_to_ptk1_names,
+    ANSICOLOR_NAMES_MAP,
+    PTK_NEW_OLD_COLOR_MAP,
+    hardcode_colors_for_win10,
+)
+
 from xonsh.color_tools import (RE_BACKGROUND, BASE_XONSH_COLORS, make_palette,
                                find_closest_color)
 from xonsh.style_tools import norm_name
 from xonsh.lazyimps import terminal256
-from xonsh.platform import os_environ
+from xonsh.platform import (
+    os_environ,
+    win_ansi_support,
+    ptk_version_info,
+    pygments_version_info,
+)
+
+from xonsh.pygments_cache import get_style_by_name
 
 
 def _command_is_valid(cmd):
@@ -281,7 +286,7 @@ def code_by_name(name, styles):
         pass
     elif 'hex' in fg:
         for p in fg.split('_'):
-            codes.append('#'+p[3:] if p.startswith('hex') else p)
+            codes.append('#' + p[3:] if p.startswith('hex') else p)
     else:
         fgtok = getattr(Color, fg.upper())
         if fgtok in styles:
@@ -292,7 +297,7 @@ def code_by_name(name, styles):
     if len(bg) == 0:
         pass
     elif bg.startswith('background_hex'):
-        codes.append('bg:#'+bg[14:])
+        codes.append('bg:#' + bg[14:])
     else:
         bgtok = getattr(Color, bg.upper())
         if bgtok in styles:
@@ -445,7 +450,12 @@ class XonshStyle(Style):
         compound = CompoundColorMap(ChainMap(self.trap, cmap, PTK_STYLE, self._smap))
         self.styles = ChainMap(self.trap, cmap, PTK_STYLE, self._smap, compound)
         self._style_name = value
-        if ON_WINDOWS:
+        # Convert new ansicolor names to old PTK1 names
+        # Can be remvoed when PTK1 support is dropped.
+        if builtins.__xonsh_shell__.shell_type != 'prompt_toolkit2':
+            for smap in [self.trap, cmap, PTK_STYLE, self._smap]:
+                smap.update(ansicolors_to_ptk1_names(smap))
+        if ON_WINDOWS and 'prompt_toolkit' in builtins.__xonsh_shell__.shell_type:
             self.enhance_colors_for_cmd_exe()
 
     @style_name.deleter
@@ -458,21 +468,24 @@ class XonshStyle(Style):
             are changed to CYAN and intense red.
         """
         env = builtins.__xonsh_env__
-        # Ensure we are not using ConEmu
-        if 'CONEMUANSI' not in env:
-            if not hasattr(pygments.style, 'ansicolors'):
-                # Auto suggest needs to be a darker shade to be distinguishable
-                # from the default color
-                self.styles[Token.AutoSuggestion] = '#444444'
-                self._smap.update(expand_gray_colors_for_cmd_exe(self._smap))
-            if env.get('INTENSIFY_COLORS_ON_WIN', False):
-                has_ansi = hasattr(pygments.style, 'ansicolors')
-                newcolors = intensify_colors_for_cmd_exe(self._smap, ansi=has_ansi)
-                self._smap.update(newcolors)
+        # Ensure we are not using ConEmu or Visual Stuio Code
+        if 'CONEMUANSI' in env or 'VSCODE_PID' in env:
+            return
+        if env.get('INTENSIFY_COLORS_ON_WIN', False):
+            if win_ansi_support():
+                newcolors = hardcode_colors_for_win10(self.styles)
+            else:
+                newcolors = intensify_colors_for_cmd_exe(self.styles)
+            self.trap.update(newcolors)
 
 
 def xonsh_style_proxy(styler):
     """Factory for a proxy class to a xonsh style."""
+    # Monky patch pygments' list of known ansi colors
+    # with the new ansi color names used by PTK2
+    # Can be removed once pygment names get fixed.
+    pygments.style.ansicolors.update(ANSICOLOR_NAMES_MAP)
+
     class XonshStyleProxy(Style):
         """Simple proxy class to fool prompt toolkit."""
 
@@ -485,112 +498,58 @@ def xonsh_style_proxy(styler):
     return XonshStyleProxy
 
 
-if hasattr(pygments.style, 'ansicolors'):
-    PTK_STYLE = {
-        Token.Menu.Completions: 'bg:#ansilightgray #ansiblack',
-        Token.Menu.Completions.Completion: '',
-        Token.Menu.Completions.Completion.Current: 'bg:#ansidarkgray #ansiwhite',
-        Token.Scrollbar: 'bg:#ansidarkgray',
-        Token.Scrollbar.Button: 'bg:#ansiblack',
-        Token.Scrollbar.Arrow: 'bg:#ansiblack #ansiwhite bold',
-        Token.AutoSuggestion: '#ansidarkgray',
-        Token.Aborted: '#ansidarkgray',
-    }
-else:
-    PTK_STYLE = {
-        Token.Menu.Completions: 'bg:#888888 #000000',
-        Token.Menu.Completions.Completion: '',
-        Token.Menu.Completions.Completion.Current: 'bg:#555555 #ffffff',
-        Token.Scrollbar: 'bg:#555555',
-        Token.Scrollbar.Button: 'bg:#000000',
-        Token.Scrollbar.Arrow: 'bg:#000000 #ffffff bold',
-        Token.AutoSuggestion: '#666666',
-        Token.Aborted: '#666666',
-    }
+PTK_STYLE = LazyObject(lambda: {
+    Token.Menu.Completions: 'bg:ansigray ansiblack',
+    Token.Menu.Completions.Completion: '',
+    Token.Menu.Completions.Completion.Current: 'bg:ansibrightblack ansiwhite',
+    Token.Scrollbar: 'bg:ansibrightblack',
+    Token.Scrollbar.Button: 'bg:ansiblack',
+    Token.Scrollbar.Arrow: 'bg:ansiblack ansiwhite bold',
+    Token.AutoSuggestion: 'ansibrightblack',
+    Token.Aborted: 'ansibrightblack',
+}, globals(), 'PTK_STYLE')
 
-if hasattr(pygments.style, 'ansicolors'):
-    XONSH_BASE_STYLE = LazyObject(lambda: {
-        Whitespace: '#ansilightgray',
-        Comment: 'underline #ansiteal',
-        Comment.Preproc: 'underline #ansibrown',
-        Keyword: 'bold #ansidarkgreen',
-        Keyword.Pseudo: 'nobold',
-        Keyword.Type: 'nobold #ansidarkred',
-        Operator: '#ansidarkgray',
-        Operator.Word: 'bold #ansipurple',
-        Name.Builtin: '#ansidarkgreen',
-        Name.Function: '#ansiblue',
-        Name.Class: 'bold #ansiblue',
-        Name.Namespace: 'bold #ansiblue',
-        Name.Exception: 'bold #ansired',
-        Name.Variable: '#ansidarkblue',
-        Name.Constant: '#ansidarkred',
-        Name.Label: '#ansiyellow',
-        Name.Entity: 'bold #ansilightgray',
-        Name.Attribute: '#ansiyellow',
-        Name.Tag: 'bold #ansidarkgreen',
-        Name.Decorator: '#ansifuchsia',
-        String: '#ansired',
-        String.Doc: 'underline',
-        String.Interpol: 'bold #ansipurple',
-        String.Escape: 'bold #ansibrown',
-        String.Regex: '#ansipurple',
-        String.Symbol: '#ansibrown',
-        String.Other: '#ansidarkgreen',
-        Number: '#ansidarkgray',
-        Generic.Heading: 'bold #ansidarkblue',
-        Generic.Subheading: 'bold #ansipurple',
-        Generic.Deleted: '#ansidarkred',
-        Generic.Inserted: '#ansigreen',
-        Generic.Error: 'bold #ansired',
-        Generic.Emph: 'underline',
-        Generic.Prompt: 'bold #ansidarkblue',
-        Generic.Output: '#ansidarkblue',
-        Generic.Traceback: '#ansidarkblue',
-        Error: '#ansired',
-        }, globals(), 'XONSH_BASE_STYLE')
-else:
-    XONSH_BASE_STYLE = LazyObject(lambda: {
-        Whitespace: "#bbbbbb",
-        Comment: "italic #408080",
-        Comment.Preproc: "noitalic #BC7A00",
-        Keyword: "bold #008000",
-        Keyword.Pseudo: "nobold",
-        Keyword.Type:   "nobold #B00040",
-        Operator:   "#666666",
-        Operator.Word:  "bold #AA22FF",
-        Name.Builtin: "#008000",
-        Name.Function: "#0000FF",
-        Name.Class: "bold #0000FF",
-        Name.Namespace: "bold #0000FF",
-        Name.Exception: "bold #D2413A",
-        Name.Variable: "#19177C",
-        Name.Constant: "#880000",
-        Name.Label: "#A0A000",
-        Name.Entity: "bold #999999",
-        Name.Attribute: "#7D9029",
-        Name.Tag: "bold #008000",
-        Name.Decorator: "#AA22FF",
-        String: "#BA2121",
-        String.Doc: "italic",
-        String.Interpol: "bold #BB6688",
-        String.Escape: "bold #BB6622",
-        String.Regex: "#BB6688",
-        String.Symbol: "#19177C",
-        String.Other: "#008000",
-        Number: "#666666",
-        Generic.Heading: "bold #000080",
-        Generic.Subheading: "bold #800080",
-        Generic.Deleted: "#A00000",
-        Generic.Inserted: "#00A000",
-        Generic.Error: "#FF0000",
-        Generic.Emph: "italic",
-        Generic.Strong: "bold",
-        Generic.Prompt: "bold #000080",
-        Generic.Output: "#888",
-        Generic.Traceback: "#04D",
-        Error: "border:#FF0000"
-    }, globals(), 'XONSH_BASE_STYLE')
+
+XONSH_BASE_STYLE = LazyObject(lambda: {
+    Whitespace: 'ansigray',
+    Comment: 'underline ansicyan',
+    Comment.Preproc: 'underline ansiyellow',
+    Keyword: 'bold ansigreen',
+    Keyword.Pseudo: 'nobold',
+    Keyword.Type: 'nobold ansired',
+    Operator: 'ansibrightblack',
+    Operator.Word: 'bold ansimagenta',
+    Name.Builtin: 'ansigreen',
+    Name.Function: 'ansibrightblue',
+    Name.Class: 'bold ansibrightblue',
+    Name.Namespace: 'bold ansibrightblue',
+    Name.Exception: 'bold ansibrightred',
+    Name.Variable: 'ansiblue',
+    Name.Constant: 'ansired',
+    Name.Label: 'ansibrightyellow',
+    Name.Entity: 'bold ansigray',
+    Name.Attribute: 'ansibrightyellow',
+    Name.Tag: 'bold ansigreen',
+    Name.Decorator: 'ansibrightmagenta',
+    String: 'ansibrightred',
+    String.Doc: 'underline',
+    String.Interpol: 'bold ansimagenta',
+    String.Escape: 'bold ansiyellow',
+    String.Regex: 'ansimagenta',
+    String.Symbol: 'ansiyellow',
+    String.Other: 'ansigreen',
+    Number: 'ansibrightblack',
+    Generic.Heading: 'bold ansiblue',
+    Generic.Subheading: 'bold ansimagenta',
+    Generic.Deleted: 'ansired',
+    Generic.Inserted: 'ansibrightgreen',
+    Generic.Error: 'bold ansibrightred',
+    Generic.Emph: 'underline',
+    Generic.Prompt: 'bold ansiblue',
+    Generic.Output: 'ansiblue',
+    Generic.Traceback: 'ansiblue',
+    Error: 'ansibrightred',
+}, globals(), 'XONSH_BASE_STYLE')
 
 
 KNOWN_COLORS = LazyObject(lambda: frozenset([
@@ -675,7 +634,7 @@ KNOWN_COLORS = LazyObject(lambda: frozenset([
     'UNDERLINE_YELLOW',
     'WHITE',
     'YELLOW',
-    ]), globals(), 'KNOWN_COLORS')
+]), globals(), 'KNOWN_COLORS')
 
 
 def _expand_style(cmap):
@@ -684,13 +643,13 @@ def _expand_style(cmap):
         if key is Color.NO_COLOR:
             continue
         _, _, key = str(key).rpartition('.')
-        cmap[getattr(Color, 'BOLD_'+key)] = 'bold ' + val
-        cmap[getattr(Color, 'UNDERLINE_'+key)] = 'underline ' + val
-        cmap[getattr(Color, 'BOLD_UNDERLINE_'+key)] = 'bold underline ' + val
+        cmap[getattr(Color, 'BOLD_' + key)] = 'bold ' + val
+        cmap[getattr(Color, 'UNDERLINE_' + key)] = 'underline ' + val
+        cmap[getattr(Color, 'BOLD_UNDERLINE_' + key)] = 'bold underline ' + val
         if val == 'noinherit':
-            cmap[getattr(Color, 'BACKGROUND_'+key)] = val
+            cmap[getattr(Color, 'BACKGROUND_' + key)] = val
         else:
-            cmap[getattr(Color, 'BACKGROUND_'+key)] = 'bg:' + val
+            cmap[getattr(Color, 'BACKGROUND_' + key)] = 'bg:' + val
 
 
 def _bw_style():
@@ -712,75 +671,31 @@ def _bw_style():
         Color.RED: 'noinherit',
         Color.WHITE: 'noinherit',
         Color.YELLOW: 'noinherit',
-        }
+    }
     _expand_style(style)
     return style
 
 
 def _default_style():
-    if hasattr(pygments.style, 'ansicolors'):
-        style = {
-            Color.BLACK: '#ansiblack',
-            Color.BLUE: '#ansidarkblue',
-            Color.CYAN: '#ansiteal',
-            Color.GREEN: '#ansidarkgreen',
-            Color.INTENSE_BLACK: '#ansidarkgray',
-            Color.INTENSE_BLUE: '#ansiblue',
-            Color.INTENSE_CYAN: '#ansiturquoise',
-            Color.INTENSE_GREEN: '#ansigreen',
-            Color.INTENSE_PURPLE: '#ansifuchsia',
-            Color.INTENSE_RED: '#ansired',
-            Color.INTENSE_WHITE: '#ansiwhite',
-            Color.INTENSE_YELLOW: '#ansiyellow',
-            Color.NO_COLOR: 'noinherit',
-            Color.PURPLE: '#ansipurple',
-            Color.RED: '#ansidarkred',
-            Color.WHITE: '#ansilightgray',
-            Color.YELLOW: '#ansibrown',
-        }
-    elif ON_WINDOWS and 'CONEMUANSI' not in os_environ:
-        # These colors must match the color specification
-        # in prompt_toolkit, so the colors are converted
-        # correctly when using cmd.exe
-        style = {
-            Color.BLACK: '#000000',
-            Color.BLUE: '#0000AA',
-            Color.CYAN: '#00AAAA',
-            Color.GREEN: '#00AA00',
-            Color.INTENSE_BLACK: '#444444',
-            Color.INTENSE_BLUE: '#4444FF',
-            Color.INTENSE_CYAN: '#44FFFF',
-            Color.INTENSE_GREEN: '#44FF44',
-            Color.INTENSE_PURPLE: '#FF44FF',
-            Color.INTENSE_RED: '#FF4444',
-            Color.INTENSE_WHITE: '#FFFFFF',
-            Color.INTENSE_YELLOW: '#FFFF44',
-            Color.NO_COLOR: 'noinherit',
-            Color.PURPLE: '#AA00AA',
-            Color.RED: '#AA0000',
-            Color.WHITE: '#888888',
-            Color.YELLOW: '#AAAA00',
-        }
-    else:
-        style = {
-            Color.BLACK: '#000000',
-            Color.BLUE: '#0000AA',
-            Color.CYAN: '#00AAAA',
-            Color.GREEN: '#00AA00',
-            Color.INTENSE_BLACK: '#555555',
-            Color.INTENSE_BLUE: '#0000FF',
-            Color.INTENSE_CYAN: '#55FFFF',
-            Color.INTENSE_GREEN: '#00FF00',
-            Color.INTENSE_PURPLE: '#FF00FF',
-            Color.INTENSE_RED: '#FF0000',
-            Color.INTENSE_WHITE: '#ffffff',
-            Color.INTENSE_YELLOW: '#FFFF55',
-            Color.NO_COLOR: 'noinherit',
-            Color.PURPLE: '#AA00AA',
-            Color.RED: '#AA0000',
-            Color.WHITE: '#aaaaaa',
-            Color.YELLOW: '#ffff00',
-        }
+    style = {
+        Color.BLACK: 'ansiblack',
+        Color.BLUE: 'ansiblue',
+        Color.CYAN: 'ansicyan',
+        Color.GREEN: 'ansigreen',
+        Color.INTENSE_BLACK: 'ansibrightblack',
+        Color.INTENSE_BLUE: 'ansibrightblue',
+        Color.INTENSE_CYAN: 'ansibrightcyan',
+        Color.INTENSE_GREEN: 'ansibrightgreen',
+        Color.INTENSE_PURPLE: 'ansibrightmagenta',
+        Color.INTENSE_RED: 'ansibrightred',
+        Color.INTENSE_WHITE: 'ansiwhite',
+        Color.INTENSE_YELLOW: 'ansibrightyellow',
+        Color.NO_COLOR: 'noinherit',
+        Color.PURPLE: 'ansimagenta',
+        Color.RED: 'ansired',
+        Color.WHITE: 'ansigray',
+        Color.YELLOW: 'ansiyellow',
+    }
     _expand_style(style)
     return style
 
@@ -831,7 +746,7 @@ def _algol_style():
         Color.RED: '#FF0000',
         Color.WHITE: '#888',
         Color.YELLOW: '#FF0000',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -855,7 +770,7 @@ def _algol_nu_style():
         Color.RED: '#FF0000',
         Color.WHITE: '#888',
         Color.YELLOW: '#FF0000',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -879,7 +794,7 @@ def _autumn_style():
         Color.RED: '#aa0000',
         Color.WHITE: '#aaaaaa',
         Color.YELLOW: '#aa5500',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -903,7 +818,7 @@ def _borland_style():
         Color.RED: '#aa0000',
         Color.WHITE: '#aaaaaa',
         Color.YELLOW: '#a61717',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -927,7 +842,7 @@ def _colorful_style():
         Color.RED: '#A00000',
         Color.WHITE: '#bbbbbb',
         Color.YELLOW: '#A60',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -951,7 +866,7 @@ def _emacs_style():
         Color.RED: '#A00000',
         Color.WHITE: '#bbbbbb',
         Color.YELLOW: '#BB6622',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -975,7 +890,7 @@ def _friendly_style():
         Color.RED: '#A00000',
         Color.WHITE: '#bbbbbb',
         Color.YELLOW: '#c65d09',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -999,7 +914,7 @@ def _fruity_style():
         Color.RED: '#ff0007',
         Color.WHITE: '#cdcaa9',
         Color.YELLOW: '#fb660a',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1023,7 +938,7 @@ def _igor_style():
         Color.RED: '#C34E00',
         Color.WHITE: '#CC00A3',
         Color.YELLOW: '#C34E00',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1047,7 +962,7 @@ def _lovelace_style():
         Color.RED: '#c02828',
         Color.WHITE: '#888888',
         Color.YELLOW: '#b85820',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1071,7 +986,7 @@ def _manni_style():
         Color.RED: '#AA0000',
         Color.WHITE: '#AAAAAA',
         Color.YELLOW: '#CC3300',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1095,7 +1010,7 @@ def _murphy_style():
         Color.RED: '#A00000',
         Color.WHITE: '#bbbbbb',
         Color.YELLOW: '#c65d09',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1119,7 +1034,7 @@ def _native_style():
         Color.RED: '#a61717',
         Color.WHITE: '#aaaaaa',
         Color.YELLOW: '#a61717',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1143,7 +1058,7 @@ def _paraiso_dark_style():
         Color.RED: '#ef6155',
         Color.WHITE: '#5bc4bf',
         Color.YELLOW: '#f99b15',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1167,7 +1082,7 @@ def _paraiso_light_style():
         Color.RED: '#2f1e2e',
         Color.WHITE: '#8d8687',
         Color.YELLOW: '#f99b15',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1191,7 +1106,7 @@ def _pastie_style():
         Color.RED: '#aa0000',
         Color.WHITE: '#bbbbbb',
         Color.YELLOW: '#aa6600',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1215,7 +1130,7 @@ def _perldoc_style():
         Color.RED: '#aa0000',
         Color.WHITE: '#a7a7a7',
         Color.YELLOW: '#cb6c20',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1239,7 +1154,7 @@ def _rrt_style():
         Color.RED: '#ff0000',
         Color.WHITE: '#87ceeb',
         Color.YELLOW: '#ff0000',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1263,7 +1178,7 @@ def _tango_style():
         Color.RED: '#a40000',
         Color.WHITE: '#f8f8f8',
         Color.YELLOW: '#8f5902',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1287,7 +1202,7 @@ def _trac_style():
         Color.RED: '#aa0000',
         Color.WHITE: '#aaaaaa',
         Color.YELLOW: '#808000',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1311,7 +1226,7 @@ def _vim_style():
         Color.RED: '#cd0000',
         Color.WHITE: '#cccccc',
         Color.YELLOW: '#cd0000',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1335,7 +1250,7 @@ def _vs_style():
         Color.RED: '#a31515',
         Color.WHITE: '#2b91af',
         Color.YELLOW: '#a31515',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1359,7 +1274,7 @@ def _xcode_style():
         Color.RED: '#C41A16',
         Color.WHITE: '#3F6E75',
         Color.YELLOW: '#836C28',
-        }
+    }
     _expand_style(style)
     return style
 
@@ -1391,7 +1306,7 @@ STYLES = LazyDict({
     'vim': _vim_style,
     'vs': _vs_style,
     'xcode': _xcode_style,
-    }, globals(), 'STYLES')
+}, globals(), 'STYLES')
 
 del (_algol_style, _algol_nu_style, _autumn_style, _borland_style, _bw_style,
      _colorful_style, _default_style, _emacs_style, _friendly_style,
@@ -1409,10 +1324,10 @@ def make_pygments_style(palette):
     for name, t in BASE_XONSH_COLORS.items():
         color = find_closest_color(t, palette)
         style[getattr(Color, name)] = '#' + color
-        style[getattr(Color, 'BOLD_'+name)] = 'bold #' + color
-        style[getattr(Color, 'UNDERLINE_'+name)] = 'underline #' + color
-        style[getattr(Color, 'BOLD_UNDERLINE_'+name)] = 'bold underline #' + color
-        style[getattr(Color, 'BACKGROUND_'+name)] = 'bg:#' + color
+        style[getattr(Color, 'BOLD_' + name)] = 'bold #' + color
+        style[getattr(Color, 'UNDERLINE_' + name)] = 'underline #' + color
+        style[getattr(Color, 'BOLD_UNDERLINE_' + name)] = 'bold underline #' + color
+        style[getattr(Color, 'BACKGROUND_' + name)] = 'bg:#' + color
     return style
 
 
@@ -1427,12 +1342,43 @@ def pygments_style_by_name(name):
     return astyle
 
 
+def _monkey_patch_pygments_codes():
+    """ Monky patch pygments' dict of console codes,
+        with new color names
+    """
+    import pygments.console
+    if 'brightblack' in pygments.console.codes:
+        # Assume that colors are already fixed in pygments
+        # for example when using pygments from source
+        return
+
+    if not getattr(pygments.console, "_xonsh_patched", False):
+        patched_codes = {}
+        for new, old in PTK_NEW_OLD_COLOR_MAP.items():
+            if old in pygments.console.codes:
+                patched_codes[new[1:]] = pygments.console.codes[old]
+        pygments.console.codes.update(patched_codes)
+        pygments.console._xonsh_patched = True
+
+
 #
 # Formatter
 #
 
 @lazyobject
 def XonshTerminal256Formatter():
+
+    if (
+        ptk_version_info()
+        and ptk_version_info() > (2, 0)
+        and pygments_version_info()
+        and (2, 2) <= pygments_version_info() < (2, 3)
+    ):
+        # Monky patch pygments' dict of console codes
+        # with the new color names used by PTK2
+        # Can be removed once pygment names get fixed.
+        _monkey_patch_pygments_codes()
+
     class XonshTerminal256FormatterProxy(terminal256.Terminal256Formatter):
         """Proxy class for xonsh terminal256 formatting that understands.
         xonsh color tokens.
