@@ -28,7 +28,8 @@ from xonsh.ansi_colors import (ansi_partial_color_format, ansi_color_style_names
 from xonsh.prompt.base import multiline_prompt
 from xonsh.tools import (print_exception, check_for_partial_string, to_bool,
                          columnize, carriage_return)
-from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_DARWIN, ON_POSIX, os_environ
+from xonsh.platform import (ON_WINDOWS, ON_CYGWIN, ON_MSYS, ON_DARWIN, ON_POSIX,
+                            os_environ)
 from xonsh.lazyimps import pygments, pyghooks, winutils
 from xonsh.events import events
 
@@ -71,7 +72,7 @@ def setup_readline():
     uses_libedit = readline.__doc__ and 'libedit' in readline.__doc__
     readline.set_completer_delims(' \t\n')
     # Cygwin seems to hang indefinitely when querying the readline lib
-    if (not ON_CYGWIN) and (not readline.__file__.endswith('.py')):
+    if (not ON_CYGWIN) and (not ON_MSYS) and (not readline.__file__.endswith('.py')):
         RL_LIB = lib = ctypes.cdll.LoadLibrary(readline.__file__)
         try:
             RL_COMPLETION_SUPPRESS_APPEND = ctypes.c_int.in_dll(
@@ -268,6 +269,43 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         self._current_indent = ''
         self._current_prompt = ''
         self._force_hide = None
+        self._complete_only_last_table = {
+            # Truth table for completions, keys are:
+            # (prefix_begs_quote, prefix_ends_quote, i_ends_quote,
+            #  last_starts_with_prefix, i_has_space)
+            (True, True, True, True, True): True,
+            (True, True, True, True, False): True,
+            (True, True, True, False, True): False,
+            (True, True, True, False, False): True,
+            (True, True, False, True, True): False,
+            (True, True, False, True, False): False,
+            (True, True, False, False, True): False,
+            (True, True, False, False, False): False,
+            (True, False, True, True, True): True,
+            (True, False, True, True, False): False,
+            (True, False, True, False, True): False,
+            (True, False, True, False, False): True,
+            (True, False, False, True, True): False,
+            (True, False, False, True, False): False,
+            (True, False, False, False, True): False,
+            (True, False, False, False, False): False,
+            (False, True, True, True, True): True,
+            (False, True, True, True, False): True,
+            (False, True, True, False, True): True,
+            (False, True, True, False, False): True,
+            (False, True, False, True, True): False,
+            (False, True, False, True, False): False,
+            (False, True, False, False, True): False,
+            (False, True, False, False, False): False,
+            (False, False, True, True, True): True,
+            (False, False, True, True, False): False,
+            (False, False, True, False, True): False,
+            (False, False, True, False, False): True,
+            (False, False, False, True, True): True,
+            (False, False, False, True, False): False,
+            (False, False, False, False, True): False,
+            (False, False, False, False, False): False,
+        }
         self.cmdqueue = collections.deque()
 
     def __del__(self):
@@ -296,11 +334,15 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         return '', line, line
 
     def _querycompletions(self, completions, loc):
-        """Returns whether or not we should show completions"""
+        """Returns whether or not we should show completions. 0 means that prefixes
+        should not be shown, 1 means that there is a common prefix among all completions
+        and they should be shown, while 2 means that there is no common prefix but
+        we are under the query limit and they should be shown.
+        """
         if os.path.commonprefix([c[loc:] for c in completions]):
-            return True
+            return 1
         elif len(completions) <= builtins.__xonsh_env__.get('COMPLETION_QUERY_LIMIT'):
-            return True
+            return 2
         msg = '\nDisplay all {} possibilities? '.format(len(completions))
         msg += '({GREEN}y{NO_COLOR} or {RED}n{NO_COLOR})'
         self.print_color(msg, end='', flush=True, file=sys.stderr)
@@ -311,24 +353,24 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         print()
         if not show_completions:
             rl_on_new_line()
-            return False
+            return 0
         w, h = shutil.get_terminal_size()
         lines = columnize(completions, width=w)
         more_msg = self.format_color('{YELLOW}==={NO_COLOR} more or '
                                      '{PURPLE}({NO_COLOR}q{PURPLE}){NO_COLOR}uit '
                                      '{YELLOW}==={NO_COLOR}')
         while len(lines) > h - 1:
-            print(''.join(lines[:h-1]), end='', flush=True, file=sys.stderr)
-            lines = lines[h-1:]
+            print(''.join(lines[:h - 1]), end='', flush=True, file=sys.stderr)
+            lines = lines[h - 1:]
             print(more_msg, end='', flush=True, file=sys.stderr)
             q = sys.stdin.read(1).lower()
             print(flush=True, file=sys.stderr)
             if q == 'q':
                 rl_on_new_line()
-                return False
+                return 0
         print(''.join(lines), end='', flush=True, file=sys.stderr)
         rl_on_new_line()
-        return False
+        return 0
 
     def completedefault(self, prefix, line, begidx, endidx):
         """Implements tab-completion for text."""
@@ -341,9 +383,31 @@ class ReadlineShell(BaseShell, cmd.Cmd):
                                                  begidx, endidx,
                                                  ctx=self.ctx)
         chopped = prefix[:-l]
-        rtn_completions = [chopped + i for i in completions]
+        if chopped:
+            rtn_completions = [chopped + i for i in completions]
+        else:
+            rtn_completions = completions
+        rtn = []
+        prefix_begs_quote = prefix.startswith("'") or prefix.startswith('"')
+        prefix_ends_quote = prefix.endswith("'") or prefix.endswith('"')
+        for i in rtn_completions:
+            i_ends_quote = i.endswith("'") or i.endswith('"')
+            last = i.rsplit(' ', 1)[-1]
+            last_starts_prefix = last.startswith(prefix)
+            i_has_space = ' ' in i
+            key = (prefix_begs_quote, prefix_ends_quote, i_ends_quote,
+                   last_starts_prefix, i_has_space)
+            rtn.append(last if self._complete_only_last_table[key] else i)
+        # return based on show completions
         show_completions = self._querycompletions(completions, endidx - begidx)
-        return rtn_completions if show_completions else []
+        if show_completions == 0:
+            return []
+        elif show_completions == 1:
+            return rtn
+        elif show_completions == 2:
+            return completions
+        else:
+            raise ValueError('query completions flag not understood.')
 
     # tab complete on first index too
     completenames = completedefault
