@@ -24,6 +24,7 @@ import codecs
 import builtins
 import itertools
 import collections
+import token
 from token import (AMPER, AMPEREQUAL, AT, CIRCUMFLEX,
                    CIRCUMFLEXEQUAL, COLON, COMMA, DEDENT, DOT, DOUBLESLASH,
                    DOUBLESLASHEQUAL, DOUBLESTAR, DOUBLESTAREQUAL, ENDMARKER, EQEQUAL,
@@ -46,22 +47,24 @@ blank_re = LazyObject(lambda: re.compile(br'^[ \t\f]*(?:[#\r\n]|$)', re.ASCII),
 #
 # token modifications
 #
-import token
-
+tok_name = tok_name.copy()
 __all__ = token.__all__ + ["COMMENT", "tokenize", "detect_encoding",
                            "NL", "untokenize", "ENCODING", "TokenInfo",
                            "TokenError", 'SEARCHPATH', 'ATDOLLAR', 'ATEQUAL',
                            'DOLLARNAME', 'IOREDIRECT']
-PY35 = PYTHON_VERSION_INFO >= (3, 5, 0)
-if PY35:
+HAS_ASYNC = (3, 5, 0) <= PYTHON_VERSION_INFO < (3, 7, 0)
+if HAS_ASYNC:
     ASYNC = token.ASYNC
     AWAIT = token.AWAIT
-    AUGASSIGN_OPS = r"[+\-*/%&@|^=<>]=?"
     ADDSPACE_TOKS = (NAME, NUMBER, ASYNC, AWAIT)
 else:
-    AUGASSIGN_OPS = r"[+\-*/%&|^=<>]=?"
     ADDSPACE_TOKS = (NAME, NUMBER)
 del token  # must clean up token
+PY35 = (3, 5, 0) <= PYTHON_VERSION_INFO
+AUGASSIGN_OPS = r"[+\-*/%&@|^=<>]=?"
+if not PY35:
+    AUGASSIGN_OPS = AUGASSIGN_OPS.replace('@', '')
+
 
 COMMENT = N_TOKENS
 tok_name[COMMENT] = 'COMMENT'
@@ -235,7 +238,7 @@ _redir_map = (
     # stdout to stderr
     'out>err', 'out>&2', '1>err', 'out>e', 'out>2', 'o>err', 'o>&2',
     '1>&2', 'o>e', '1>e', 'o>2', '1>2',
-    )
+)
 IORedirect = group(group(*_redir_map), '{}>>?'.format(group(*_redir_names)))
 _redir_check = set(_redir_map)
 _redir_check = {'{}>'.format(i) for i in _redir_names}.union(_redir_check)
@@ -771,9 +774,9 @@ def _tokenize(readline, encoding):
                         stashed = tok
                         continue
 
-                    if token == 'def' and (stashed and
-                                           stashed.type == NAME and
-                                           stashed.string == 'async'):
+                    if HAS_ASYNC and token == 'def' and \
+                            (stashed and stashed.type == NAME and
+                             stashed.string == 'async'):
                         async_def = True
                         async_def_indent = indents[-1]
 
