@@ -8,7 +8,7 @@ import functools
 import importlib
 import importlib.util
 
-from xonsh.tools import print_color
+from xonsh.tools import print_color, unthreadable
 
 
 @functools.lru_cache(1)
@@ -51,8 +51,8 @@ def prompt_xontrib_install(names):
     print('The following xontribs are enabled but not installed: \n'
           '   {xontribs}\n'
           'To install them run \n'
-          '    pip install {packages}'.format(xontribs=' '.join(names),
-                                              packages=' '.join(packages)))
+          '    xpip install {packages}'.format(xontribs=' '.join(names),
+                                               packages=' '.join(packages)))
 
 
 def update_context(name, ctx=None):
@@ -78,11 +78,11 @@ def xontrib_metadata():
     return md
 
 
-def _load(ns):
-    """load xontribs"""
+def xontribs_load(names, verbose=False):
+    """Load xontribs from a list of names"""
     ctx = builtins.__xonsh_ctx__
-    for name in ns.names:
-        if ns.verbose:
+    for name in names:
+        if verbose:
             print('loading xontrib {0!r}'.format(name))
         update_context(name, ctx=ctx)
     if update_context.bad_imports:
@@ -90,6 +90,11 @@ def _load(ns):
         del update_context.bad_imports
 
 
+def _load(ns):
+    """load xontribs"""
+    xontribs_load(ns.names, verbose=ns.verbose)
+
+
 def _list(ns):
     """Lists xontribs."""
     meta = xontrib_metadata()
@@ -118,7 +123,7 @@ def _list(ns):
         for d in data:
             name = d['name']
             lname = len(name)
-            s += "{PURPLE}" + name + "{NO_COLOR}  " + " "*(nname - lname)
+            s += "{PURPLE}" + name + "{NO_COLOR}  " + " " * (nname - lname)
             if d['installed']:
                 s += '{GREEN}installed{NO_COLOR}      '
             else:
@@ -154,9 +159,10 @@ def _create_xontrib_parser():
 _MAIN_XONTRIB_ACTIONS = {
     'load': _load,
     'list': _list,
-    }
+}
 
 
+@unthreadable
 def xontribs_main(args=None, stdin=None):
     """Alias that loads xontribs"""
     if not args or (args[0] not in _MAIN_XONTRIB_ACTIONS and
